function [Uxy,xB,yB,con,dimsg,rtimes] = collocation_2d(n,H,wfct,f) 
% solves -Delta u = f on D: wfct>0, D subseteq (0,1)^2
% grid width 1/H, degree n 
% Uxy: solution on xy-grid
% con, dimsg: condition and dimension of linear system
% rtimes.sys, .ext, .sol, .total: computing times

% CwBS-Programs (Version: 1.0)
% test version: for educational use and illustration purposes only
% -> see TermsOfUse for details

Uxy=[]; con=NaN; dimsg=NaN; rtimes.sys=NaN; rtimes.ext=NaN; rtimes.sol=NaN; rtimes.total=NaN;

if n>5, display('degree n to large'), return, end
load(sprintf('collocation_data_2d_n%1d',n));
dim = size(CD.E);

display('classification'); tic;
% bandwidth: 1+2*bw points within a B-spline support; even n+1, odd n
bw = fix(n/2);

% weight function, domain, and points within supports
t = (-(n-1)/2-bw:H+(n-1)/2+bw)/H; 
[x,y] = ndgrid(t,t);
[w,wx,wy,wxx,wyy] = wfct(x,y);

% collocation points 
indB = 1+bw:H+n+bw; 
xB = x(indB,indB); yB = y(indB,indB); wB = w(indB,indB); 

% index lists for inner and outer B-splines: (k1,k2) ~ I(m,1:2) <-> Bindex(k1,k2) ~ m
Bindex=zeros(H+n);
Btype = wB>0;
IL=find(Btype); mi=length(IL); Bindex(IL)=1:mi;

% arrays (dim = (n+1)^2) of inner B-splines with centers (xe(m),ye(m)) 
eI = true(H,H);
for s1=1:n+1, for s2=1:n+1
  eI = eI & Btype(s1:s1+H-1,s2:s2+H-1); 
end, end
if ~any(eI(:)), display('grid width 1/H too large'), return, end  
[xe,ye] = ndgrid((1/2:H)/H);
xe =xe(eI); ye =ye(eI);

% index lists for outer B-splines: (k1,k2) ~ J(m,1:2) <-> Bindex(k1,k2) ~ m
for k1=-bw:bw, for k2=-bw:bw
  Btype=Btype | w(k1+indB,k2+indB)>0;
end, end
JL=find(Btype &~(wB>0)); mj=length(JL); Bindex(JL)=1:mj;
[J1,J2]=ind2sub(size(Btype),JL);
rtimes.sys=toc;

% extension Matrix T
display('extension'); tic; 
T=spalloc(mi,mj,mj*(n+1).^2);
for m=1:mj, 
  % closest array center (xej,yej) to outer B-spline ~ j = J(m,1:2)
  j1 = J1(m); j2 = J2(m);
  d = (xe-xB(j1,j2)).^2+(ye-yB(j1,j2)).^2;
  [tmp,ind] = min(d(:)); 
  xej = xe(ind(1)); yej = ye(ind(1)); 
  IJ = round([xej, yej]*H+1/2); 
  D = [j1,j2]-IJ-n/2;
  D0 = 1+(D>=0); D1 = 1+fix(abs(D)); 
  if max(D1) > dim(3), display('distance of j too large'), return, end
  partind=Bindex(IJ(1):IJ(1)+n,IJ(2):IJ(2)+n);    
  T(partind(:),m)=CD.E(:,D1(1),D1(2),D0(1),D0(2));
end
rtimes.ext=toc;

% collocation matrix G stored in band format 
display('assembling'); tic;
G = zeros(H+n,H+n,2*bw+1,2*bw+1); 
for s1=-bw:bw, for s2=-bw:bw
  S1=1+bw+s1; S2=1+bw+s2; 
  G(:,:,1+bw-s1,1+bw-s2) = ... 
    -H*H*(CD.b20(S1,S2)+CD.b02(S1,S2)).*wB ... 
  -CD.b00(S1,S2).*(wxx(indB,indB)+wyy(indB,indB)) ...
  -2*H*(CD.b10(S1,S2).*wx(indB,indB)+CD.b01(S1,S2).*wy(indB,indB));
  end, end
b00 = CD.b00; clear CD
F = f(xB,yB);

% conversion of array and extension 
SG=array2matrix(G); clear G;
SG=SG(IL,IL)+SG(IL,JL)*T';
rtimes.sys=rtimes.sys+toc;

% solution of linear system 
display('solving'); tic;
SU = SG\F(IL); 

% weighted B-spline coefficients
display('extension');
U = zeros(H+n,H+n); 
U(IL) = SU; U(JL)= T'*SU;
rtimes.sol=toc;
rtimes.total=rtimes.sys+rtimes.ext+rtimes.sol;

% condition of SG
dimsg=size(SG,1);  
if dimsg < 1000, display('condest'); con = condest(SG); else con = NaN; end

% evaluation at grid points
display('evaluation')
Uxy = zeros(H+n+2*bw,H+n+2*bw); 
for s1=1:1+2*bw, for s2=1:1+2*bw
  ind1 = (s1:s1+H+n-1); ind2 = (s2:s2+H+n-1); 
  Uxy(ind1,ind2) = Uxy(ind1,ind2) + ... 
    U.*max(w(ind1,ind2),0)*b00(s1,s2);
end, end

Uxy=Uxy(indB,indB);


function GM = array2matrix(G)
% system parameters
bw=(size(G,3)-1)/2;
dim=size(G,1);
c=dim^2;
[K1,K2]=ndgrid(-bw:bw);
I=K1+dim*K2; clear K1 K2
I1=(ones((2*bw+1).^2,1)*(1:c))';
I2=I1+(I(:)*ones(1,c))'; clear I
I2=reshape(I2,size(G));
for k=1:bw
  I2(k,:,bw+1-k,:)=-1;
  I2(dim+1-k,:,bw+1+k,:)=-1;
end
ind=find(I2>0 & I2 <=c); I1=I1(ind); I2=I2(ind);
GM=sparse(I1,I2,G(ind),c,c);
