
function [Uxyz,x,y,z,con,dimsg,rtimes] = collocation_3d(n,H,wfct,f) 
% solves -Delta u = f on D: wfct>0, D subseteq (0,1)^3
% grid width 1/H, degree n 
% Uxyz: solution of xyz-grid
% con, dimsg: condition and dimension of linear system
% rtimes.sys, .ext, .sol, .total: computing times

% CwBS-Programs (Version: 1.0)
% test version: for educational use and illustration purposes only
% -> see TermsOfUse for details

Uxyz=[]; con=NaN; dimsg=NaN; rtimes.sys=NaN; rtimes.ext=NaN; rtimes.sol=NaN; rtimes.total=NaN;

if n>5, display('degree n to large'), return, end
load(sprintf('collocation_data_3d_n%1d',n));
dim = size(CD.E);

display('classification'); tic;
% bandwidth: 1+2*bw points within a B-spline support; even n+1, odd n
bw = fix(n/2);

% weight function, domain, and points within supports
t = (-(n-1)/2-bw:H+(n-1)/2+bw)/H; 
[x,y,z] = ndgrid(t,t,t);
[w,wx,wy,wz,wxx,wyy,wzz] = wfct(x,y,z);

% collocation points 
indB = 1+bw:H+n+bw; 
xB = x(indB,indB,indB); yB = y(indB,indB,indB); zB = z(indB,indB,indB); wB = w(indB,indB,indB);

% index lists for inner B-splines: (k1,k2,k3) ~ I(m,1:3) <-> Bindex(k1,k2,k3) ~ m
Bindex=zeros(H+n,H+n,H+n);
Btype = wB>0;
IL=find(Btype); mi=length(IL); Bindex(IL)=1:mi;

% arrays (dim = (n+1)^3) of inner B-splines with centers (xe(m),ye(m),ze(m)) 
eI = true(H,H,H);
for s1=1:n+1, for s2=1:n+1, for s3=1:n+1
  eI = eI & Btype(s1:s1+H-1,s2:s2+H-1,s3:s3+H-1); 
end, end, end
if ~any(eI(:)), display('grid width 1/H too large'), return, end
[xe,ye,ze] = ndgrid((1/2:H)/H);xe=xe(eI); ye=ye(eI); ze=ze(eI);

% index lists for outer B-splines: (k1,k2,k3) ~ J(m,1:3) <-> Bindex(k1,k2,k3) ~ m
for k1=-bw:bw, for k2=-bw:bw, for k3=-bw:bw
  Btype=Btype | w(k1+indB,k2+indB,k3+indB)>0;
end, end, end
JL=find(Btype & ~(wB>0)); mj=length(JL); Bindex(JL)=1:mj;
[J1,J2,J3]=ind2sub([H+n,H+n,H+n],JL); 
rtimes.sys=toc;

% extension matrix T 
display('extension'); tic;
T=spalloc(mi,mj,mi+mj*(n+1).^3);
for m=1:mj, 
  % closest array center (xej,yej,zej) to outer B-spline ~ j = J(m,1:3)
  j1=J1(m); j2=J2(m); j3=J3(m); 
  d = (xe-xB(j1,j2,j3)).^2+(ye-yB(j1,j2,j3)).^2+(ze-zB(j1,j2,j3)).^2;
  [tmp,ind] = min(d(:)); 
  xej = xe(ind(1)); yej = ye(ind(1)); zej = ze(ind(1));
  IJ = round([xej, yej, zej]*H+1/2); 
  D = [j1,j2,j3]-IJ-n/2;
  D0 = 1+(D>=0); D1 = 1+fix(abs(D)); 
  if max(D1) > dim(4), display('distance of j too large'), return, end
  partind=Bindex(IJ(1):IJ(1)+n,IJ(2):IJ(2)+n,IJ(3):IJ(3)+n);    
  T(partind(:),m)=CD.E(:,D1(1),D1(2),D1(3),D0(1),D0(2),D0(3));  
end
rtimes.ext=toc;

% collocation matrix G stored in band format 
display('assembling'); tic;
G = zeros(H+n,H+n,H+n,2*bw+1,2*bw+1,2*bw+1); 
for s1=-bw:bw, for s2=-bw:bw, for s3=-bw:bw
  S1=1+bw+s1; S2=1+bw+s2; S3=1+bw+s3;
  G(:,:,:,1+bw-s1,1+bw-s2,1+bw-s3) = ... 
    -H*H*(CD.b200(S1,S2,S3)+CD.b020(S1,S2,S3)+CD.b002(S1,S2,S3)).*wB ... 
  -CD.b000(S1,S2,S3).*(wxx(indB,indB,indB)+wyy(indB,indB,indB)+wzz(indB,indB,indB)) ...
  -2*H*(CD.b100(S1,S2,S3).*wx(indB,indB,indB)+CD.b010(S1,S2,S3).*wy(indB,indB,indB)+CD.b001(S1,S2,S3).*wz(indB,indB,indB));
end, end, end
b000 = CD.b000; clear CD
F = f(xB,yB,zB);

% conversion of array and extension 
SG=array2matrix(G); clear G;
SG=SG(IL,IL)+SG(IL,JL)*T';
rtimes.sys=rtimes.sys+toc;

% solution of linear system 
display('solving'); tic;
SU = SG\F(IL);

% weighted B-spline coefficients
display('extension');
U = zeros(H+n,H+n,H+n); 
U(IL) = SU; U(JL)= T'*SU;
rtimes.sol=toc;
rtimes.total=rtimes.sys+rtimes.ext+rtimes.sol;

% condition of SG
dimsg=size(SG,1);
if dimsg < 1000, display('condest'); con = condest(SG); else con = NaN; end

% evaluation at grid points
display('evaluation')
Uxyz = zeros(H+n+2*bw,H+n+2*bw,H+n+2*bw); 
for s1=1:1+2*bw, for s2=1:1+2*bw, for s3=1:1+2*bw 
  ind1 = s1:s1+H+n-1; ind2 = s2:s2+H+n-1; ind3 = s3:s3+H+n-1; 
  Uxyz(ind1,ind2,ind3) = Uxyz(ind1,ind2,ind3) + ... 
    U.*max(w(ind1,ind2,ind3),0)*b000(s1,s2,s3);
end, end, end

function GM = array2matrix(G)
% system parameters
bw=(size(G,4)-1)/2;
dim=size(G,1);
c=dim^3;
[K1,K2,K3]=ndgrid(-bw:bw);
I=K1+dim*K2+dim^2*K3; clear K1 K2 K3
I1=(ones((2*bw+1).^3,1)*(1:c))';
I2=I1+(I(:)*ones(1,c))'; clear I
I2=reshape(I2,size(G));
for k=1:bw
  I2(k,:,:,bw+1-k,:,:)=-1;
  I2(dim+1-k,:,:,bw+1+k,:,:)=-1;
  I2(:,k,:,:,bw+1-k,:)=-1;
  I2(:,dim+1-k,:,:,bw+1+k,:)=-1;
end
ind=find(I2>0 & I2 <=c); I1=I1(ind); I2=I2(ind);
GM=sparse(I1,I2,G(ind),c,c);
