function collocation_data_2d(n,J_MAX) 
% precomputes extension coefficients and B-spline values for degree n
% J_MAX: maximal distance of arrays of inner B-spline centers

% CwBS-Programs (Version: 1.0)
% test version: for educational use and illustration purposes only
% -> see TermsOfUse for details

% extension coefficients 
dim = fix(J_MAX+n/2)+1;
CD.E = zeros(n+1,n+1,dim,dim,2,2);
t = [0:n];
for i=0:n;
  L = zeros(1,n+1); L(i+1)=1; 
  p = polyfit(t,L,n);   
  for j=-J_MAX:n+J_MAX 
    D0 = ((j-n/2)>=0)+1; D1 = fix(abs(j-n/2))+1;
    EU(i+1,D1,D0) = round(polyval(p,j));
  end
end
for j1=1:dim, for j2=1:dim
  for s1=1:2, for s2=1:2
	CD.E(:,:,j1,j2,s1,s2) = EU(:,j1,s1)*EU(:,j2,s2)';
  end, end
end, end
CD.E = reshape(CD.E,[(n+1)^2,dim,dim,2,2]);

% B-spline values
bw = fix(n/2);
t = [-bw:bw]+(n+1)/2; t = t(:); 
dim = length(t)*[1,1];

b = bspl(t,n); 
db = bspl(t,n-1)-bspl(t-1,n-1);
ddb = bspl(t,n-2)-2*bspl(t-1,n-2)+bspl(t-2,n-2);

CD.b00 = b(:)*b(:)'; 
CD.b10 = db(:)*b(:)'; 
CD.b01 = b(:)*db(:)'; 
CD.b20 = ddb(:)*b(:)'; 
CD.b02 = b(:)*ddb(:)'; 

save(sprintf('collocation_data_2d_n%1d',n),'CD');

% B-spline evaluation
function b = bspl(x,n)

if n==0
   b = 0*x; 
   ind = find(x>=0 & x<1);
   b(ind) = 1;
elseif n==1
   b = 0*x; 
   ind = find(x>=0 & x<1); 
   b(ind) = x(ind); 
   ind = find(x>=1 & x<2); 
   b(ind) = 2-x(ind); 
else
   b = (x/n).*bspl(x,n-1) + ((n+1-x)/n).*bspl(x-1,n-1);
end

