function collocation_data_3d(n,J_MAX) 
% precomputes extension coefficients and B-spline values for degree n
% J_MAX: maximal distance of arrays of inner b-spline centers

% CwBS-Programs (Version: 1.0)
% test version: for educational use and illustration purposes only
% -> see TermsOfUse for details

Uxy=[]; con=NaN; dimsg=NaN; rtimes.sys=NaN; rtimes.ext=NaN; rtimes.sol=NaN; rtimes.total=NaN;

% extension coefficients 
dim = fix(J_MAX+n/2)+1;
CD.E = zeros(n+1,n+1,n+1,dim,dim,dim,2,2,2);
t = [0:n];
for i=0:n;
  L = zeros(1,n+1); L(i+1)=1; 
  p = polyfit(t,L,n);   
  for j=-J_MAX:n+J_MAX 
  D0 = ((j-n/2)>=0)+1; D1 = fix(abs(j-n/2))+1;
  EU(i+1,D1,D0) = round(polyval(p,j));
  end
end
for j1=1:dim, for j2=1:dim, for j3=1:dim
  for s1=1:2, for s2=1:2, for s3=1:2
    E = EU(:,j1,s1)*EU(:,j2,s2)';
    CD.E(:,:,:,j1,j2,j3,s1,s2,s3) = reshape(E(:)*EU(:,j3,s3)',n+1,n+1,n+1);
  end, end, end
end, end, end
CD.E = reshape(CD.E,[(n+1)^3,dim,dim,dim,2,2,2]);

% B-spline values
bw = fix(n/2);
t = [-bw:bw]+(n+1)/2; t = t(:); 
dim = length(t)*[1,1,1];
 b = bspl(t,n); 
db = bspl(t,n-1)-bspl(t-1,n-1);
ddb = bspl(t,n-2)-2*bspl(t-1,n-2)+bspl(t-2,n-2);
  
b000 = b(:)*b(:)'; CD.b000 = reshape(b000(:)*b(:)',dim);
b100 = db(:)*b(:)'; CD.b100 = reshape(b100(:)*b(:)',dim);
b010 = b(:)*db(:)'; CD.b010 = reshape(b010(:)*b(:)',dim);
b001 = b(:)*b(:)'; CD.b001 = reshape(b001(:)*db(:)',dim);
b200 = ddb(:)*b(:)'; CD.b200 = reshape(b200(:)*b(:)',dim);
b020 = b(:)*ddb(:)'; CD.b020 = reshape(b020(:)*b(:)',dim);
b002 = b(:)*b(:)'; CD.b002 = reshape(b002(:)*ddb(:)',dim);

save(sprintf('collocation_data_3d_n%1d',n),'CD');

% B-spline evaluation
function b = bspl(x,n)

if n==0
   b = 0*x; 
   ind = find(x>=0 & x<1);
   b(ind) = 1;
elseif n==1
   b = 0*x; 
   ind = find(x>=0 & x<1); 
   b(ind) = x(ind); 
   ind = find(x>=1 & x<2); 
   b(ind) = 2-x(ind); 
else
   b = (x/n).*bspl(x,n-1) + ((n+1-x)/n).*bspl(x-1,n-1);
end

