function example_2d()
% illustrates the use of collocation_2d.m

% CwBS-Programs (Version: 1.0)
% test version: for educational use and illustration purposes only
% -> see TermsOfUse for details

% degree n and grid width 1/H
n = 5; H = 64;
 
% generate data file if it does not yet exist
if exist(sprintf('collocation_data_2d_n%1d.mat',n),'file')==0;
  display('preprocessing')
  % dist_bd: maximal distance of arrays of inner B-spline centers
  dist_bd = 16; 
  collocation_data_2d(n,dist_bd);
end

display('start collocation')
% numerical solution Uxy of -Delta u_exact = f_exact 
% on the domain D: wfct(x,y)>0 which MUST be contained in (0,1)^2 
[Uxy,x,y,con,dim,rtimes] = collocation_2d(n,H,@wfct,@f_exact); 

% assign zero values outside the domain 
uxy = u_exact(x,y).*(wfct(x,y)>0); 

% error
ErrMax = norm(Uxy(:)-uxy(:),inf)/norm(uxy(:),inf)
ErrL2 = norm(Uxy(:)-uxy(:),2)/norm(uxy(:),2)

% condition, dimension, and total solution time
condition = con, dimension = dim, total_time = rtimes.total

% visualize solution 
surf(x,y,Uxy,Uxy) 

%%%%% sub-functions

function [w,wx,wy,wxx,wyy] = wfct(x,y) 
% weight function for a domain D: w>0 in (0,1)^2

domain = 'disc';
%domain = 'shovel';
%domain = 'bezier';
switch domain
case 'disc'
   w = 1-(2*x-1).^2-(2*y-1).^2;
   wx = -4*(2*x-1); wy = -4*(2*y-1);  
   wxx = 0*w-8; wyy = 0*w-8;  

case 'shovel'
   p = 2*x-1; q = 2.25*y-1/4;
   w = 1-p.^2-(q+p.^2-1).^2;
   wx = -8*p.*q-8*p.^3+4*p; wy = -4.5*(q+p.^2-1);
   wxx = -16*q-48*p.^2+8; wyy = 0*y-10.125;  

case 'bezier'
   % Bezier coefficients
   C = {-1 0 -8 0 -1; 0 16 -10 16 0; -8 -10 4 -10 -8; 0 16 -10 16 0; -1 0 -8 0 -1};
   n = size(C,1)-1;
   % evaluation weights
   p00 = (1-x).*(1-y); p01 = (1-x).*y; p10 = x.*(1-y); p11 = x.*y;
   % de Casteljau algorithm
   for k=0:n-3
      for kx=1:n-k, for ky=1:n-k 
         C{kx,ky} = p00.*C{kx,ky} + p01.*C{kx,ky+1} + p10.*C{kx+1,ky} + p11.*C{kx+1,ky+1};
      end, end
   end
   wxx = n*(n-1)*((1-y).^2.*(C{3,1}-2*C{2,1}+C{1,1}) + ... 
      2*y.*(1-y).*(C{3,2}-2*C{2,2}+C{1,2}) + ... 
      y.^2.*(C{3,3}-2*C{2,3}+C{1,3}));
   wyy = n*(n-1)*((1-x).^2.*(C{1,3}-2*C{1,2}+C{1,1}) + ... 
      2*x.*(1-x).*(C{2,3}-2*C{2,2}+C{2,1}) + ... 
      x.^2.*(C{3,3}-2*C{3,2}+C{3,1}));
   for kx=1:2, for ky=1:2 
      C{kx,ky} = p00.*C{kx,ky} + p01.*C{kx,ky+1} + p10.*C{kx+1,ky} + p11.*C{kx+1,ky+1};
   end, end
   wx = n*((1-y).*(C{2,1}-C{1,1}) + y.*(C{2,2}-C{1,2}));
   wy = n*((1-x).*(C{1,2}-C{1,1}) + x.*(C{2,2}-C{2,1}));
   w = p00.*C{1,1}+p01.*C{1,2}+p10.*C{2,1}+p11.*C{2,2};
end

function u = u_exact(x,y)
% exact solution 
w = wfct(x,y);
u = exp(w)-1;

function f = f_exact(x,y)
% right hand side (computed from exact solution)
[w,wx,wy,wxx,wyy] = wfct(x,y);
f = -exp(w).*(wx.^2+wy.^2+wxx+wyy);

