function example_3d()
% illustrates the use of collocation_3d.m

% CwBS-Programs (Version: 1.0)
% test version: for educational use and illustration purposes only
% -> see TermsOfUse for details

% degree n and grid width 1/H
n = 3; H = 60;

% generate data file if it does not yet exist
if exist(sprintf('collocation_data_3d_n%1d.mat',n),'file')==0; 
  display('preprocessing')
  % dist_bd: maximal distance of arrays of inner B-spline centers
  dist_bd = 16; 
  collocation_data_3d(n,dist_bd);
end

display('start collocation')
% numerical solution Uxyz of -Delta u_exact = f_exact 
% on the domain D: wfct(x,y,z)>0 which MUST be contained in (0,1)^3 
[Uxyz,x,y,z,con,dim,rtimes] = collocation_3d(n,H,@wfct,@f_exact); 

% assign zero values outside the domain 
uxyz = u_exact(x,y,z).*(wfct(x,y,z)>0); 

% error
ErrMax = norm(Uxyz(:)-uxyz(:),inf)/norm(uxyz(:),inf)
ErrL2 = norm(Uxyz(:)-uxyz(:),2)/norm(uxyz(:),2)

% condition, dimension, and total solution time
condition = con, dimension = dim, total_time = rtimes.total

% visualize solution
slice(y,x,z,permute(Uxyz,[2,1,3]),1/2,1/2,1/2)

%%%%% sub-functions

function [w,wx,wy,wz,wxx,wyy,wzz] = wfct(x,y,z) 
% weight function for a domain D: w>0 in (0,1)^3

%domain = 'ball';
domain = 'algebraic';
%domain = 'rvachev';
switch domain 
case 'ball'
   w = 1-(2*x-1).^2-(2*y-1).^2-(2*z-1).^2;
   wx = -4*(2*x-1); wy = -4*(2*y-1); wz = -4*(2*z-1);
   wxx = 0*w-8; wyy = 0*w-8; wzz = 0*w-8;

 case 'algebraic'
  x=5*x-5/2;y=5*y-5/2;z=5*z-5/2;
  e1=(x.^2 + y.^2/3 -1);
  e2=(x.^2/4 + y.^2/4 -1);
  w = 1/4-(e1.*e2).^2-z.^2;
  wx = -4.*x.*e1.*e2.*(5*e2+5/4.*e1);
  wxx = - 20*e1.*e2.*(5*e2+5/4.*e1)-...
	40.*x.^2.*e2.*(5*e2+5/4.*e1)-...
	10.*x.^2.*e1.*(5*e2+5/4.*e1)-...
	100.*x.^2.*e1.*e2;
  wy = -4.*y.*e1.*e2.*(5/3.*e2+5/4.*e1);
  wyy = -20.*e1.*e2.*(5/3.*e2+5/4.*e1)-...
	40/3.*y.^2.*e2.*(5/3.*e2+5/4.*e1)-...
	10.*y.^2.*e1.*(5/3.*e2+5/4.*e1)-...
	100/3.*y.^2.*e1.*e2;
  wz = -10.*z;
  wzz=-50.*ones(size(z));
 
 case 'rvachev'
   w1 = 1-(2*x-1).^2-(2*z-1).^2;
   w1x = -8*x+4; w1y = 0*y; w1z = -8*z+4;
   w1xx = 0*x-8; w1yy = w1y; w1zz = 0*z-8;
   w2 = 1-(2*y-1).^2-(2*z-1).^2;
   w2x = 0*x; w2y = -8*y+4; w2z = -8*z+4;
   w2xx = w2x; w2yy = 0*y-8; w2zz = 0*z-8;

   s = sqrt(w1.^2+w2.^2);
   w = w1+w2-s; 
   wx = w1x+w2x-(w1.*w1x+w2.*w2x)./s;
   wy = w1y+w2y-(w1.*w1y+w2.*w2y)./s;
   wz = w1z+w2z-(w1.*w1z+w2.*w2z)./s;
   wxx = w1xx+w2xx-(w1x.^2+w1.*w1xx+w2x.^2+w2.*w2xx)./s+(w1.*w1x+w2.*w2x).^2./s.^3;
   wyy = w1yy+w2yy-(w1y.^2+w1.*w1yy+w2y.^2+w2.*w2yy)./s+(w1.*w1y+w2.*w2y).^2./s.^3;
   wzz = w1zz+w2zz-(w1z.^2+w1.*w1zz+w2z.^2+w2.*w2zz)./s+(w1.*w1z+w2.*w2z).^2./s.^3;
end

function u = u_exact(x,y,z)
% exact solution 
w = wfct(x,y,z);
u = w.*exp(x+y.*z);

function f = f_exact(x,y,z)
% right hand side (computed from exact solution)
[w,wx,wy,wz,wxx,wyy,wzz] = wfct(x,y,z);
% ux = (wx+w)*e, wy = (wy+w*z)*e, wz = (wz+w*y)*e 
f = -(wxx+2*wx+w + wyy+2*wy.*z+w.*z.^2 + wzz+2*wz.*y+w.*y.^2).*exp(x+y.*z);

