function [Ux,x] = interpolation_1d(H,n,fct,M)
% interpolates fct at B-spline centers on [0,1]  
% grid width 1/H, degree n, M evaluation points per knot interval 
% Ux: spline values at x

% CwBS-Programs (Version: 1.0)
% test version: for educational use and illustration purposes only
% -> see TermsOfUse for details

% B-spline values: 1+2*bw points within a support; even n+1, odd n
bw = fix(n/2);
B = bspline((n+1)/2+[0:bw]',n);

% interpolation matrix
dim = H+n-2*bw;
A = zeros(dim,dim); 
for d=-bw:bw
  A = A + B(1+abs(d))*diag(ones(dim-abs(d),1),d);
end

% extension (left and right)
for k=1:bw
  I = [1:n+1]; 
  for j=-bw+k:0
    A(k,I) = A(k,I) + eij(I,j)*B(1+k-j);
  end
end   
for k=dim-bw+1:dim 
  I = [dim-n:dim];
  for j=dim+1:k+bw
    A(k,I) = A(k,I) + eij(I,j)*B(1+j-k);
  end
end

% right hand side
xF = [(1-n)/2+bw:(n-1)/2+H-bw]'/H; 
F = fct(xF);

% solution of linear system 
V = A\F;

% B-spline coefficients
for j=-bw+1:0
   U(j+bw) = eij([1:n+1],j)*V(1:n+1);
end
U(bw+1:bw+dim) = V; 
for j=dim+1:dim+bw
   U(j+bw) = eij([dim-n:dim],j)*V(dim-n:dim);
end
U = U(:);

% evaluation of the spline
B = bspline([1:(n+1)*M]/M,n);
Ux = zeros((H+2*n)*M,1); 
for k=1:(n+1)*M
   Ux(k:M:k+(H+n-1)*M) = Ux(k:M:k+(H+n-1)*M) + B(k)*U; 
end
Ux = Ux(n*M:(H+n)*M); 
x = [0:H*M]'/(H*M);

function E = eij(I,j)
% extension coefficients

n = length(I)-1;
E = ones(1,n+1); 
for i=1:n+1
  E(i) = 1; 
  for k=1:n+1
    if k~=i
      E(i) = E(i) * (j-I(k))/(I(i)-I(k));
    end
  end
end

function b = bspline(t,n) 
% values of the b-spline of degree n at t
if n==1
  b = max(t,0)-2*max(t-1,0)+max(t-2,0); 
else
  b = (t.*bspline(t,n-1)+(n+1-t).*bspline(t-1,n-1))/n;
end

